package eos.io.grapher;

import java.awt.BasicStroke;
import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.DatasetRenderingOrder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.DeviationRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.data.xy.YIntervalSeries;
import org.jfree.data.xy.YIntervalSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;

import eos.util.In;

/**
 * The Grapher plots XY-graphs and XY-deviation-graphs. To use it:
 * <p>
 * 1. Prepare input files. The first line of the file must be the column titles.
 * Column 0 (the first column) must be the time step. All entries must be comma
 * delimited with no space. Here is a sample input:
 * <p>
 * <tt>Step,nPrice</tt><br>
 * <tt>0,0.30</tt><br>
 * <tt>5,0.36</tt><br>
 * <tt>10,0.47</tt><br>
 * <tt>15,0.62</tt><br>
 * <tt>20,0.79</tt><br>
 * <tt>25,0.97</tt><br>
 * <tt>30,1.14</tt><br>
 * <tt>35,1.29</tt><br>
 * <tt>40,1.25</tt><br>
 * <tt>45,1.18</tt>
 * <p>
 * The easiest way to generate input files is to use the Printers.
 * <p>
 * 2. Create a new <tt>Grapher</tt>. See
 * {@link #Grapher(String title, String yAxisName)}.
 * <p>
 * 3. Add data series to the graph. A maximum of 6 series are allowed. See
 * {@link #addSeries(String seriesName, String fileName, int col)} and
 * {@link #addDevSeries(String seriesName, String fileName, int col, int lowCol, int highCol)}.
 * <p>
 * 4. Either use {@link #plot()} to show the plot in a pop-up window or use
 * {@link #export(String fileName)} to export it to a PNG file.
 * <p>
 * For <tt>export()</tt>, if you omit the file name or provide a simple file
 * name, the file will be saved in a folder called "output". If on the other
 * hand, you specify a directory in the file name, the file will be saved in
 * your specified directory.
 * 
 */
public class Grapher {
	/**
	 * maximal number of data series allowed
	 */
	private static final int MAX_SERIES_COUNT = 6;

	private static final Color[] colors = { new Color(255, 0, 0),
			new Color(0, 0, 255), new Color(0, 255, 0), new Color(194, 166, 1),
			new Color(68, 0, 136), new Color(2, 200, 140), };

	/**
	 * fill colors for deviation spread
	 */
	private static final Color[] fillColors = { new Color(255, 180, 180),
			new Color(180, 180, 255), new Color(180, 255, 180),
			new Color(250, 250, 170), new Color(192, 130, 255),
			new Color(196, 255, 248) };

	/**
	 * XY data set
	 */
	private XYSeriesCollection dataset1 = null;

	/**
	 * deviation data set
	 */
	private YIntervalSeriesCollection dataset2 = null;

	/**
	 * number of data series
	 */
	private int seriesCount;

	/**
	 * title of the graph
	 */
	private String title;

	/**
	 * the graph frame
	 */
	private XYChart graph = null;

	/**
	 * the chart object
	 */
	private JFreeChart chart;

	/**
	 * name of the y-axis
	 */
	private String yAxisName;

	private class XYChart extends ApplicationFrame {
		private static final long serialVersionUID = 1L;

		public XYChart(String title, XYDataset dataset1, XYDataset dataset2) {
			super(title);
			chart = createChart(title, dataset1, dataset2);
			ChartPanel chartPanel = new ChartPanel(chart);
			chartPanel.setPreferredSize(new java.awt.Dimension(1024, 768));
			setContentPane(chartPanel);
		}

		private JFreeChart createChart(String title, XYDataset dataset1,
				XYDataset dataset2) {
			XYPlot plot = new XYPlot();
			NumberAxis yAxis = new NumberAxis(yAxisName);
			NumberAxis xAxis = new NumberAxis("Step");
			xAxis.setAutoRangeIncludesZero(false);
			yAxis.setAutoRangeIncludesZero(true);
			plot.setDomainAxis(xAxis);
			plot.setRangeAxis(yAxis);

			int index = 0;
			int colorIndex = 0;

			if (dataset1 != null && dataset1.getSeriesCount() > 0) {
				XYLineAndShapeRenderer renderer1 = new XYLineAndShapeRenderer();
				renderer1
						.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
				int count = dataset1.getSeriesCount();
				for (int i = 0; i < count; i++) {
					renderer1.setSeriesLinesVisible(i, true);
					renderer1.setSeriesShapesVisible(i, false);
					renderer1.setSeriesStroke(i, new BasicStroke(3.0f));
					renderer1.setSeriesPaint(i, colors[colorIndex]);
					colorIndex++;
				}
				plot.setDataset(index, dataset1);
				plot.setRenderer(index, renderer1);
				index++;
			}

			if (dataset2 != null && dataset2.getSeriesCount() > 0) {
				DeviationRenderer renderer2 = new DeviationRenderer();
				renderer2
						.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
				renderer2.setAlpha(0.8f);
				int count = dataset2.getSeriesCount();
				for (int i = 0; i < count; i++) {
					renderer2.setSeriesLinesVisible(i, true);
					renderer2.setSeriesShapesVisible(i, false);
					renderer2.setSeriesStroke(i, new BasicStroke(3.0f));
					renderer2.setSeriesPaint(i, colors[colorIndex]);
					renderer2.setSeriesFillPaint(i, fillColors[colorIndex]);
					colorIndex++;
				}
				plot.setDataset(index, dataset2);
				plot.setRenderer(index, renderer2);
				index++;
			}

			plot.setDatasetRenderingOrder(DatasetRenderingOrder.FORWARD);
			return new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot,
					true);

		}

		public void exportToPNG(FileOutputStream out) throws IOException {
			ChartUtilities.writeChartAsPNG(out, chart, 1025, 768);
		}
	}

	/**
	 * Create a new <tt>Grapher</tt>.
	 * 
	 * @param title
	 *            title of the graph
	 *            <p>
	 * @param yAxisName
	 *            y-axis name; the x-axis is the time step
	 */
	public Grapher(String title, String yAxisName) {
		seriesCount = 0;
		this.title = title;
		this.yAxisName = yAxisName;
	}

	/**
	 * Add a data series to the graph. Entries in the input must be comma
	 * delimited (with no space). The first line must be the column titles.
	 * Column 0 (the first column) must be the time step. By default, data are
	 * assumed to be in column 1. If not, you need to specify the column number.
	 * Here is a sample input:
	 * <p>
	 * <tt>Step,nPrice</tt><br>
	 * <tt>0,0.30</tt><br>
	 * <tt>5,0.36</tt><br>
	 * <tt>10,0.47</tt><br>
	 * <tt>15,0.62</tt><br>
	 * <tt>20,0.79</tt><br>
	 * <tt>25,0.97</tt><br>
	 * <tt>30,1.14</tt><br>
	 * <tt>35,1.29</tt><br>
	 * <tt>40,1.25</tt><br>
	 * <tt>45,1.18</tt>
	 * <p>
	 * The easiest way to prepare input files is to use the Printers provided.
	 * All Printers have a <tt>getFileName()</tt> method, which you may call to
	 * get the file name. Multi-column Printers (those with flags) have a
	 * <tt>getCol(int flag)</tt> method which you may use to get the column
	 * number.
	 * <p>
	 * e.g. Say you have necessity price printer set up and has produced an output
	 * file. You can add a necessity price series to <tt>Grapher priceGrapher</tt> by
	 * calling
	 * <p>
	 * <tt>priceGrapher.addSeries("necessity price", nPricePrinter.getFileName());</tt>
	 * <p>
	 * 
	 * @param seriesName
	 *            name of the series
	 *            <p>
	 * @param fileName
	 *            name of the input file
	 *            <p>
	 * @param col
	 *            column that contains the series, column number starts from 0.
	 *            If <tt>col</tt> is omitted, it will be taken to be 1.
	 */
	public void addSeries(String seriesName, String fileName, int col) {
		if (seriesCount >= MAX_SERIES_COUNT)
			throw new RuntimeException("Too many series");
		In in = new In(fileName);
		String line = in.readLine();
		XYSeries series = new XYSeries(seriesName);
		while (!in.isEmpty()) {
			line = in.readLine();
			String[] data = line.split(",");
			int step = Integer.parseInt(data[0]);
			double value = Double.parseDouble(data[col]);
			series.add(step, value);
		}
		in.close();

		if (dataset1 == null)
			dataset1 = new XYSeriesCollection();
		dataset1.addSeries(series);
		seriesCount++;
	}

	/**
	 * Add a data series to the graph. See
	 * {@link #addSeries(String seriesName, String fileName, int col)}
	 * 
	 */
	public void addSeries(String seriesName, String fileName) {
		addSeries(seriesName, fileName, 1);
	}

	/**
	 * Add a deviation data series to the graph. A deviation series displays a
	 * spread around the trend line (see the sample output below).
	 * <p>
	 * The input file must have 4 columns. Column 0 (the first column) must be
	 * the time step. Column 1 is the trend line (or average line). Column 2 is
	 * the lower bound (the bottom edge of the spread). Column 3 is the upper
	 * bound (the top edge of the spread). If the these 3 columns are not
	 * columns 1, 2, 3, or they are not in this order, you need to specify their
	 * column numbers. All entries in the input must be comma delimited (with no
	 * space). Here is a sample input:
	 * <p>
	 * <tt>Step,Avg_SimpleLaborer_Money_Stock,Min_SimpleLaborer_Money_Stock,Max_SimpleLaborer_Money_Stock</tt>
	 * <br>
	 * <tt>0,60.146,60.0,60.975</tt><br>
	 * <tt>5,59.778,55.651,64.489</tt><br>
	 * <tt>10,58.612,52.729,62.906</tt><br>
	 * <tt>15,57.648,50.441,69.807</tt><br>
	 * <tt>20,56.754,40.334,69.722</tt><br>
	 * <tt>25,56.501,37.882,73.368</tt><br>
	 * <tt>30,58.268,36.126,76.568</tt>
	 * <p>
	 * The easiest way to generate input files is to use the Printers. The
	 * <tt>StatsPrinter.STD</tt> flag is extremely handy. It prints the average,
	 * the minimum and the maximum. You may use <tt>getFileName()</tt> of
	 * Printers to get the file name and <tt>getCol(int flag)</tt> to get the
	 * column number.
	 * <p>
	 * 
	 * @param seriesName
	 *            name of the series
	 *            <p>
	 * @param fileName
	 *            name of the input file
	 *            <p>
	 * @param col
	 *            column number of the average values
	 *            <p>
	 * @param lowCol
	 *            column number of the lower bounds
	 *            <p>
	 * @param highCol
	 *            column number of the upper bounds
	 *            <p>
	 * 
	 */
	public void addDevSeries(String seriesName, String fileName, int col,
			int lowCol, int highCol) {
		if (seriesCount >= MAX_SERIES_COUNT)
			throw new RuntimeException("Too many series");
		In in = new In(fileName);
		String line = in.readLine();
		YIntervalSeries series = new YIntervalSeries(seriesName);
		while (!in.isEmpty()) {
			line = in.readLine();
			String[] data = line.split(",");
			int step = Integer.parseInt(data[0]);
			double value = Double.parseDouble(data[col]);
			double min = Double.parseDouble(data[lowCol]);
			double max = Double.parseDouble(data[highCol]);
			series.add(step, value, min, max);
		}
		in.close();

		if (dataset2 == null)
			dataset2 = new YIntervalSeriesCollection();
		dataset2.addSeries(series);
		seriesCount++;
	}

	/**
	 * Add a deviation data series to the graph. See
	 * {@link #addDevSeries(String seriesName, String fileName, int col, int lowCol, int highCol)}
	 */
	public void addDevSeries(String seriesName, String fileName) {
		addDevSeries(seriesName, fileName, 1, 2, 3);
	}

	/**
	 * Plot the graph in a pop-up window, with zooming tools
	 */
	public void plot() {
		graph = new XYChart(title, dataset1, dataset2);
		graph.pack();
		RefineryUtilities.centerFrameOnScreen(graph);
		graph.setVisible(true);
	}

	/**
	 * Export the graph to a PNG file
	 * 
	 * @param fileName
	 *            name of the file
	 */
	public void export(String fileName) {
		if (graph == null) {
			graph = new XYChart(title, dataset1, dataset2);
		}
		if (!fileName.endsWith(".png"))
			fileName = fileName + ".png";
		int i = fileName.lastIndexOf('\\');
		i = Math.max(i, fileName.lastIndexOf('/'));
		String dirName;
		if (i == -1) {
			dirName = "output";
			fileName = dirName + "/" + fileName;
		} else
			dirName = fileName.substring(0, i);
		File file = new File(dirName);
		if (!file.exists())
			file.mkdirs();
		try {
			FileOutputStream out = new FileOutputStream(fileName);
			graph.exportToPNG(out);
			out.close();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

	}
}
