package eos.simulation;

import eos.agent.laborer.*;
import eos.agent.firm.CFirm;
import eos.agent.firm.EFirm;
import eos.agent.firm.NFirm;
import eos.market.*;
import eos.util.StdRandom;
import eos.economy.*;
import eos.io.grapher.Grapher;
import eos.io.printer.*;

/**
 * Simulation (heterogeneous case)
 * 
 * @author zhihongx
 * 
 */

/**
 * <b>Guidelines for creating a simulation</b>
 * <p>
 * Please follow the steps below. The order is important.
 * <p>
 * 1. Create the markets and add them into the economy. The markets need to be
 * created first because the constructors of the agents reference the markets.
 * Here is an example of creating a labor market and adding it to the economy.
 * <p>
 * <tt>LaborMarket lMkt = new LaborMarket();<br>
 * Economy.addMarket(lMkt);</tt>
 * <p>
 * 2. Create the firms. Capital firms need to be created before other firms,
 * because the constructors of consumer goods firms require reference to an
 * array of capital firms.
 * <p>
 * 3. Add the firms to the economy. For instance, here is how to add the
 * necessity firms:
 * <p>
 * <tt>for (int i = 0; i < NUM_NFIRMS; i++) Economy.addAgent(nFirms[i]);</tt>
 * <p>
 * 4. Create laborers and add them to the economy.
 * <p>
 * 5. Clear the labor market by calling <tt>lMkt.clear()</tt>. This is to allow
 * firms to get labor before the start of the first time step.
 * <p>
 * 6. Create printers and add them to the economy. Several printers have been
 * provided in <tt>eos.io.printer</tt>, but feel free to create more customized
 * printers.
 * <p>
 * 7. Run the simulation by calling <tt>Economy.run(NUM_STEP)</tt>.
 * <p>
 * 8. Clean up the printers by calling <tt>Economy.cleanUpPrinters()</tt>
 * <p>
 * 9. Plot graphs.
 * <p>
 */
public class Simulation2 {

	/*************** constants **********************************/

	// number of steps between two printer outputs
	private static final int STEP_SIZE = 50;

	// number of steps to run
	private static final int NUM_STEP = 10000;

	private static final int NUM_LABORERS = 450;
	private static final int NUM_EFIRMS = 10;
	private static final int NUM_NFIRMS = 10;

	private static final double MIN_INIT_E_PRICE = 0.1;
	private static final double MAX_INIT_E_PRICE = 5;
	private static final double MIN_INIT_N_PRICE = 0.1;
	private static final double MAX_INIT_N_PRICE = 5;

	private static final double EFIRM_INIT_CHECKING = 100;
	private static final double EFIRM_INIT_SAVINGS = -1000;
	private static final double EFIRM_INIT_OUTPUT = 40;
	private static final double EFIRM_INIT_WAGEBUDGET = 100;
	private static final int EFIRM_INIT_CAPITAL = 30;

	private static final double NFIRM_INIT_CHECKING = 100;
	private static final double NFIRM_INIT_SAVINGS = -1000;
	private static final double NFIRM_INIT_OUTPUT = 50;
	private static final double NFIRM_INIT_WAGEBUDGET = 100;
	private static final int NFIRM_INIT_CAPITAL = 30;

	private static final double CFIRM_INIT_WAGEBUDGET = 500;
	private static final double CFIRM_INIT_CHECKING = CFIRM_INIT_WAGEBUDGET;
	private static final double CFIRM_INIT_SAVINGS = 0;

	private static final double LABORER_INIT_E = 0;
	private static final double LABORER_INIT_CHECKING = 0;
	private static final double LABORER_INIT_SAVINGS = 100;
	private static final double LABORER_INIT_SAVINGS_RATE = 0.9;

	/************************************************************/

	public static void main(String[] args) {

		// set the seed for the pseudorandom number generator
		StdRandom.setSeed(2345);

		/* Create and add markets */
		ConsumerGoodMarket eMkt = new ConsumerGoodMarket("Enjoyment",
				MIN_INIT_E_PRICE, MAX_INIT_E_PRICE);
		ConsumerGoodMarket nMkt = new ConsumerGoodMarket("Necessity",
				MIN_INIT_N_PRICE, MAX_INIT_N_PRICE);
		LaborMarket lMkt = new LaborMarket();
		CapitalMarket cMkt = new CapitalMarket();
		Economy.addMarket(lMkt);
		Economy.addMarket(eMkt);
		Economy.addMarket(nMkt);
		Economy.addMarket(cMkt);

		/* Create and add firms */
		CFirm cFirm = new CFirm(CFIRM_INIT_CHECKING, CFIRM_INIT_SAVINGS,
				CFIRM_INIT_WAGEBUDGET);
		CFirm[] cFirms = new CFirm[1];
		cFirms[0] = cFirm;

		EFirm[] eFirms = new EFirm[NUM_EFIRMS];
		for (int i = 0; i < NUM_EFIRMS; i++) {
			double initSavings = StdRandom.uniform(EFIRM_INIT_SAVINGS * 1.1,
					EFIRM_INIT_SAVINGS * 0.9);
			eFirms[i] = new EFirm(EFIRM_INIT_CHECKING, initSavings,
					EFIRM_INIT_OUTPUT, EFIRM_INIT_WAGEBUDGET,
					EFIRM_INIT_CAPITAL, cFirms);
		}

		NFirm[] nFirms = new NFirm[NUM_NFIRMS];
		for (int i = 0; i < NUM_NFIRMS; i++) {
			double initSavings = StdRandom.uniform(NFIRM_INIT_SAVINGS * 1.1,
					NFIRM_INIT_SAVINGS * 0.9);
			nFirms[i] = new NFirm(NFIRM_INIT_CHECKING, initSavings,
					NFIRM_INIT_OUTPUT, NFIRM_INIT_WAGEBUDGET,
					NFIRM_INIT_CAPITAL, cFirms);
		}

		Economy.addAgent(cFirm);
		for (int i = 0; i < NUM_NFIRMS; i++)
			Economy.addAgent(nFirms[i]);
		for (int i = 0; i < NUM_EFIRMS; i++)
			Economy.addAgent(eFirms[i]);

		/* Create and add laborers */
		Laborer[] laborers = new Laborer[NUM_LABORERS];
		for (int i = 0; i < NUM_LABORERS; i++) {
			double initN = StdRandom.gaussian(15, 3);
			double initSavings = StdRandom.uniform(LABORER_INIT_SAVINGS * 0.9,
					LABORER_INIT_SAVINGS * 1.1);
			laborers[i] = new Laborer(LABORER_INIT_E, initN,
					LABORER_INIT_CHECKING, initSavings,
					LABORER_INIT_SAVINGS_RATE);
			Economy.addAgent(laborers[i]);
		}

		/* clear labor market */
		lMkt.clear();

		/* Create and add printers */
		LaborersPrinter laborersPrt = new LaborersPrinter("Laborer", STEP_SIZE,
				laborers);
		Economy.addPrinter(laborersPrt);

		ConsumerMktPricePrinter ePricePrt = new ConsumerMktPricePrinter(
				"EPrice", STEP_SIZE, eMkt);
		Economy.addPrinter(ePricePrt);
		ConsumerMktVolPrinter eVolPrt = new ConsumerMktVolPrinter("EVol",
				STEP_SIZE, eMkt);
		Economy.addPrinter(eVolPrt);
		FirmsPrinter eFirmsPrt = new FirmsPrinter("EFirms", STEP_SIZE, eFirms);
		Economy.addPrinter(eFirmsPrt);

		ConsumerMktPricePrinter nPricePrt = new ConsumerMktPricePrinter(
				"NPrice", STEP_SIZE, nMkt);
		Economy.addPrinter(nPricePrt);
		ConsumerMktVolPrinter nVolPrt = new ConsumerMktVolPrinter("NVol",
				STEP_SIZE, nMkt);
		Economy.addPrinter(nVolPrt);
		FirmsPrinter nFirmsPrt = new FirmsPrinter("NFirms", STEP_SIZE, nFirms);
		Economy.addPrinter(nFirmsPrt);

		BankPrinter bankPrt = new BankPrinter("Bank", STEP_SIZE);
		Economy.addPrinter(bankPrt);

		/* Run simulation */
		Economy.run(NUM_STEP);
		Economy.cleanUpPrinters();

		/* Create plots */
		Grapher icPlot = new Grapher("Laborer Income", "$");
		icPlot.addSeries("avg laborer wage", laborersPrt.getFileName(), 1);
		icPlot.addSeries("avg laborer income", laborersPrt.getFileName(), 2);
		icPlot.export("1_laborer_income.png");
		icPlot.plot();

		Grapher sPlot = new Grapher("Savings & Loans", "$");
		sPlot.addSeries("laborer total savings", laborersPrt.getFileName(), 5);
		sPlot.addSeries("nFirm totalloans", nFirmsPrt.getFileName(), 3);
		sPlot.addSeries("eFirm totalloans", eFirmsPrt.getFileName(), 3);
		sPlot.export("2_savings&loans.png");
		sPlot.plot();

		Grapher cPlot = new Grapher("Consumption", "$");
		cPlot.addSeries("Avg N Consumption", laborersPrt.getFileName(), 9);
		cPlot.addSeries("Avg E Consumption", laborersPrt.getFileName(), 10);
		cPlot.export("3_consumption.png");
		cPlot.plot();

		Grapher pPlot = new Grapher("Prices", "$");
		pPlot.addSeries("Necessity Price", nPricePrt.getFileName());
		pPlot.addSeries("Enjoyment Price", ePricePrt.getFileName());
		pPlot.export("4_prices.png");
		pPlot.plot();

		Grapher vPlot = new Grapher("Volumes", "volume");
		vPlot.addSeries("Necessity volume", nFirmsPrt.getFileName(), 2);
		vPlot.addSeries("Enjoyment volume", eFirmsPrt.getFileName(), 2);
		vPlot.export("5_volumes.png");
		vPlot.plot();

		Grapher irPlot = new Grapher("Interest Rate", "rate");
		irPlot.addSeries("loan IR", bankPrt.getFileName(), 1);
		irPlot.addSeries("deposit IR", bankPrt.getFileName(), 3);
		irPlot.export("6_interest_rate.png");
		irPlot.plot();

		Grapher nsPlot = new Grapher("NStock", "stock");
		nsPlot.addSeries("laborers NStock", laborersPrt.getFileName(), 7);
		nsPlot.export("7_necessity_stock.png");
		nsPlot.plot();

		Grapher pfPlot = new Grapher("Profit", "profit");
		pfPlot.addSeries("NFirms Profit", nFirmsPrt.getFileName(), 7);
		pfPlot.addSeries("EFirms Profit", eFirmsPrt.getFileName(), 7);
		pfPlot.export("8_profit.png");
		pfPlot.plot();

		Grapher mpPlot = new Grapher("Marginal Profit", "marginal profit");
		mpPlot.addSeries("NFirms marginal profit", nFirmsPrt.getFileName(), 9);
		mpPlot.addSeries("EFirms marginal profit", eFirmsPrt.getFileName(), 9);
		mpPlot.export("9_marginal_profit.png");
		mpPlot.plot();

		Grapher lbPlot = new Grapher("Labor", "labor");
		lbPlot.addSeries("NFirms labor", nFirmsPrt.getFileName(), 8);
		lbPlot.addSeries("EFirms labor", eFirmsPrt.getFileName(), 8);
		lbPlot.export("10_labor.png");
		lbPlot.plot();

		Grapher nfPlot = new Grapher("NFirms Revenue and Cost", "$");
		nfPlot.addSeries("NFirms total revenue", nFirmsPrt.getFileName(), 1);
		nfPlot.addSeries("NFirms total cost", nFirmsPrt.getFileName(), 10);
		nfPlot.addSeries("NFirms total labor cost", nFirmsPrt.getFileName(), 11);
		nfPlot.addSeries("NFirms total capital cost", nFirmsPrt.getFileName(),
				12);
		nfPlot.export("11_nfirms.png");
		nfPlot.plot();

		Grapher efPlot = new Grapher("EFirms Revenue and Cost", "$");
		efPlot.addSeries("EFirms total revenue", eFirmsPrt.getFileName(), 1);
		efPlot.addSeries("EFirms total cost", eFirmsPrt.getFileName(), 10);
		efPlot.addSeries("EFirms total labor cost", eFirmsPrt.getFileName(), 11);
		efPlot.addSeries("EFirms total capital cost", eFirmsPrt.getFileName(),
				12);
		efPlot.export("12_efirms.png");
		efPlot.plot();
	}
}
